#include "waypoint_mfd.h"
using namespace std;

string parser2::getErrorString()
{
	return errorstring;
}

double parser2::myatof(string s)
{
	
	double d = atof(s.c_str());
	if (d == 0 && s.at(0) != '0')
	{
		errorOut("error converting " + s + " to double.");		
		return -1;
	}
	return d;
}

vector<waypoint> parser2::getRoute(string fileName)
{
	oapiWriteLog ("Waypoint MFD: Start parsing file...");
	fileString = readFileLocal(fileName.c_str());
	fileString = this->replaceAll();
	vector<point> points = getPoints();
	runways = getRunways(points);
	meineroute = getRoute(points,runways);
	oapiWriteLog ("Waypoint MFD: Parsing completed!");
	return meineroute;
}

int parser2::myatoi(string s)
{
	int i = atoi(s.c_str());
	if (i == 0 && s.at(0) != '0')
	{
		errorOut("error converting " + s + " to int.");		
		return -1;
	}
	return i;
}

vector<waypoint> parser2::getRoute(vector<point> points, vector<runway> runways)
{
	
	oapiWriteLog("getRoute 1");
	vector<waypoint> waypoints;
	string str = getElements("WAYPOINTS");
	if (str.size() == 0)
	{
		errorOut("Error parsing file!\nEND_WAYPOINTS not found after BEGIN_WAYPOINTS?\n");
		return waypoints;
	}
	oapiWriteLog("getRoute 2");
	vector<string> ptbuff = split(str,"\n");
	for (unsigned int ip =0; ip<ptbuff.size(); ip++)
	{
		oapiWriteLog(".1");
		waypoint wp;
		//parse comment
		int startComment = ptbuff.at(ip).find("\"");
		if (startComment > 0)
		{
			int endComment = ptbuff.at(ip).find("\"",startComment+1);
			if (endComment > startComment)
			{
				wp.comment = ptbuff.at(ip).substr(startComment+1,endComment-startComment-1);
				//printf("gut!");
			}
			
		}
		else
			wp.comment = "";
		vector<string> elementStr = split(ptbuff.at(ip)," ");

		unsigned int id = unsigned int(myatoi(elementStr.at(1)));	
		if (parsererror) return waypoints;

		if (elementStr.at(2) == "TURN") wp.type =  0;
		else if (elementStr.at(2) == "LOOP") wp.type = 1;
		else if (elementStr.at(2) == "TAKEOFF") wp.type=  2;
		else if (elementStr.at(2) == "LAND") wp.type=  3;
		oapiWriteLog(".2");
		wp.radius = myatof (elementStr.at(3));
		if (parsererror) return waypoints;
		bool error = false;
		oapiWriteLog(".3");
		if (elementStr.size() > 4)
		{
			
			wp.alt = myatof(elementStr.at(4));
			if (parsererror) return waypoints;
			wp.speed = myatof(elementStr.at(5));
			if (parsererror) return waypoints;
		}
		else
		{
			wp.alt = 0;
			wp.speed = 0;
		}
		oapiWriteLog(".4");
		if (elementStr.at(0) == "RUNWAY")
		{
			oapiWriteLog(".5");
			if (runways.size() <= id) 
			{
				this->errorOut("Error: Runway in route not found!");
				error = true;
			}
			else
			{
				wp.lat = (runways.at(id).center.lat);
				wp.lon = (runways.at(id).center.lon);			
			}	
		}
		else if (elementStr.at(0) == "POINT")
		{
			oapiWriteLog(".6");
			if (points.size() <= id) 
			{
				this->errorOut("Error: Point in route not found!");
				error = true;
			}
			else
			{
				wp.lat = MYMATH::degToRad(points.at(id).lat);
				wp.lon = MYMATH::degToRad(points.at(id).lon);
			}
			oapiWriteLog(".7");
		}
		oapiWriteLog(".8");
		if (!error) waypoints.push_back(wp);
		oapiWriteLog(".9");
	}
	return waypoints;
}

void parser2::errorOut(string errormessage)
{
	parsererror = true;
	errorstring = errormessage;
	oapiWriteLog ("Waypoint MFD Log:");
	printf(errormessage.c_str());
	if (errormessage.length()<499)
	{
		char c[500];
		strcpy(c,errormessage.c_str());
		oapiWriteLog (c);
	}
	else
	{
		oapiWriteLog ("Waypoint MFD: Error log to lonng!");
	}
}
vector<runway> parser2::getRunways(vector<point> points)
{
	oapiWriteLog ("Parsing Runways...");
	vector<runway> runways;
	string runwaysStr = getElements("RUNWAYS");
	if (runwaysStr.size() == 0)
	{
		errorOut("Error parsing file!\nEND_RUNWAYS not found after BEGIN_RUNWAYS?\n");
		return runways;
	}
	vector<string> ptbuff = split(runwaysStr,"\n");
	for (int ip =0; ip<ptbuff.size(); ip++)
	{
		vector<string> elementStr = split(ptbuff.at(ip)," ");
		int ip1 = myatoi(elementStr.at(0));
		if (parsererror) return runways;
		int ip2 = myatoi(elementStr.at(1));
		if (parsererror) return runways;
		if (ip1 == -1|| ip2 == -1) 
			return runways;
		point p1,p2;
		p1.lon = MYMATH::degToRad(points.at(ip1).lon);
		p1.lat = MYMATH::degToRad(points.at(ip1).lat);
		p2.lon = MYMATH::degToRad(points.at(ip2).lon);
		p2.lat = MYMATH::degToRad(points.at(ip2).lat);
		runway rw;
		rw.p1 = p1;
		rw.p2 = p2;
		double delta = 0;
		if (rw.p1.lat > rw.p2.lat)
		{
			delta = rw.p1.lat - rw.p2.lat;
			rw.center.lat = rw.p2.lat + (delta / 2);
		}
		if (rw.p1.lat < rw.p2.lat)
		{
			delta = rw.p2.lat - rw.p1.lat;
			rw.center.lat = rw.p1.lat + (delta / 2);
		}

		if (rw.p1.lon > rw.p2.lon)
		{
			delta = rw.p1.lon - rw.p2.lon;
			rw.center.lon = rw.p2.lon + (delta / 2);
		}
		if (rw.p1.lon < rw.p2.lon)
		{
			delta = rw.p2.lon - rw.p1.lon;
			rw.center.lon = rw.p1.lon + (delta / 2);
		}
		runways.push_back(rw);
	}
	oapiWriteLog ("Ok!");
	return runways;
}
vector<point> parser2::getPoints()
{
	oapiWriteLog ("Parsing Points...");
	vector<point> points;
	string allpoints = getElements("POINTS");
	if (allpoints.size() == 0)
	{
		errorOut("Error parsing file!\nEND_POINTS not found after BEGIN_POINTS?\n");
		return points;
	}
	oapiWriteLog ("..start spilt 1..");

	vector<string> ptbuff = split(allpoints,"\n");

	oapiWriteLog ("..end spilt 1..");
	vector<runway> runways;
	oapiWriteLog (".1..");
	for (int ip =0; ip<ptbuff.size(); ip++)
	{
		oapiWriteLog ("..");
		point p;
		oapiWriteLog ("..start spilt 2..");
		vector<string> elementStr = split(ptbuff.at(ip)," ");
		oapiWriteLog ("..end spilt 2..");
		p.lon = myatof(elementStr.at(0));
		if (parsererror) return points;
		p.lat = myatof(elementStr.at(1));
		if (parsererror) return points;
		points.push_back(p);
	}
	oapiWriteLog ("Ok!");
	return points;
}
void parser2::readFile(std::string fileName)
{
	oapiWriteLog ("reading file...");
	this->readFileLocal(fileName.c_str());
	oapiWriteLog ("...file read");
}
parser2::parser2()
{
	parsererror = false;
	this->errorstring = "";
}

string parser2::replaceAll()
{
	fileString = this->replace(fileString,"\t"," ");
	fileString = this->replace(fileString,"  "," ");
	fileString = this->replace(fileString,"\n ","\n");
	fileString = this->replace(fileString,"\n\n","\n");	
	return fileString;
}

string parser2::readFileLocal(const char* filename)
{
	try
	{
		string fstring;
		ifstream fin(filename);
		if (fin == NULL)
		{
			throw 0;
		}
		string zeile;
		getline(fin, zeile);
		
		fstring = fstring + zeile;
		do 
		{
			getline(fin, zeile);
			zeile = zeile.substr(0,zeile.find("#"));
			fstring = fstring + zeile + "\n";
		} while (!fin.eof());
		return fstring;
	}
	catch(...)
	{
		return "";
	}
}

string parser2::replace(string source,string replace, string by)
{
	int pos = source.find(replace); 
	while (pos >= 0)
	{	
		source = source.replace(pos,replace.length(),by);
		pos = source.find(replace); 
	}
	return source;
}
/*
vector<string> parser2::split(string sourceString,string delimitter) 
{
	oapiWriteLog("sourceString is");
	char tmpbuf[256];
	sprintf(tmpbuf,"%s",sourceString.c_str());
	oapiWriteLog(tmpbuf);
	oapiWriteLog("delimitter is");
	sprintf(tmpbuf,"%c",delimitter.c_str());
	oapiWriteLog(tmpbuf);


	oapiWriteLog ("spit1");
	std::vector<std::string> elements;
	oapiWriteLog ("spit2");
	char * pch;
	oapiWriteLog ("spit3");
	char * buffer1 = new char[sourceString.length()];
	oapiWriteLog ("spit4");
	strcpy(buffer1,sourceString.c_str());
	oapiWriteLog ("spit5");

	char logbuffer[256];
	sprintf(logbuffer,"%i",delimitter.length());
	oapiWriteLog("length is:");

	oapiWriteLog(logbuffer);
	char * buffer2 = new char[delimitter.length()];
	//char * buffer2 = new char[256];
	oapiWriteLog ("spit6");
	strcpy(buffer2,delimitter.c_str());
	oapiWriteLog ("spit7");
	pch = strtok (buffer1,buffer2);
	oapiWriteLog ("spit8");
	if (pch != NULL) 
	{
		oapiWriteLog ("spit9");
		std::string buffer3(pch);
		oapiWriteLog ("spit10");
		elements.push_back(buffer3);
		oapiWriteLog ("spit11");
	}
	oapiWriteLog ("spit12");
	while(pch != NULL)
	{
		oapiWriteLog ("spit13");
		pch = strtok (NULL,buffer2);
		oapiWriteLog ("spit14");
		if (pch != NULL)
		{
			oapiWriteLog ("spit15");
			std::string buffer4(pch);
			oapiWriteLog ("spit16");
			elements.push_back(buffer4);
			oapiWriteLog ("spit17");
		}
		oapiWriteLog ("spit18");
	}
	oapiWriteLog ("spit19");
	return elements;
}
*/
vector<string> parser2::split(string str, string separator)
{
	vector<string> results;
    int found;
    found = str.find_first_of(separator);
    while(found != string::npos){
        if(found > 0){
            results.push_back(str.substr(0,found));
        }
        str = str.substr(found+1);
        found = str.find_first_of(separator);
    }
    if(str.length() > 0)
	{
        results.push_back(str);
    }
	return results;

}

string parser2::getElements(string name)
{
	try
	{
		string nodename = name;
		string nodenameStart = "BEGIN_" + nodename;
		string nodenameEnd = "END_" + nodename;

		int startTest = fileString.find(nodenameStart);
		int EndTest = fileString.find(nodenameEnd);

		int start = fileString.find(nodenameStart) + nodenameStart.length();
		int end = fileString.find(nodenameEnd);

		if (EndTest<startTest) throw 1;
		if (startTest==-1) throw 1;
		if (EndTest==-1 && start >= 0) throw 1;

		string elements = fileString.substr(start,end-start);
		if (elements.find("\n") == 0)
			elements = elements.substr(1,elements.length());
		
		int test1 =  elements.find("\n");
		if(elements.at(elements.length()-1) == '\n')
			elements = elements.substr(0,elements.length()-1);
		return elements;
	}
	catch(...)
	{
		this->errorOut("Error: BEGIN_" + name + " or END_"  + name + "not found");
		return "";
	}
}