#define STRICT
#define ORBITER_MODULE
#include "waypoint_mfd.h"
#define STRICT
extern std::vector<MFDHANDLE> gdata;
int mode;

class Mymod : public oapi::Module 
{
	public:
		Mymod (HINSTANCE hDLL): Module (hDLL) {}
		void clbkPostStep(double simt,double simdt,double mjd);
		void clbkSimulationStart(oapi::Module::RenderMode mode);
		void clbkSimulationEnd();
	private:
		void checkWaypointReached(unsigned int currentIndex);
		double getGearAlt(VESSEL* vessel);
};

double Mymod::getGearAlt(VESSEL* vessel)
{
		double Alt = vessel->GetAltitude();
		VECTOR3 tp1,tp2,tp3;
		vessel->GetTouchdownPoints(tp1,tp2,tp3);
		double kleinste = tp1.y;
		if (tp2.y < kleinste)  kleinste = tp2.y;
		if (tp3.y < kleinste)  kleinste = tp3.y;
		return Alt + kleinste;
}

void Mymod::clbkPostStep(double simt,double simdt,double mjd)
{
	if (gdata.size()>=1)
	{
		for (unsigned int i = 0; i<gdata.size(); i++)
		{
			checkWaypointReached(i);
		}
	}
}

void Mymod::clbkSimulationStart(oapi::Module::RenderMode rmode)
{
	
}
void Mymod::clbkSimulationEnd()
{
	gdata.clear();
}
void Mymod::checkWaypointReached(unsigned int currentIndex)
{	
	if (gdata.at(currentIndex).route.size() > 0)
	{
		double alt = 0, spd = 0;
		if (gdata.at(currentIndex).vessel != NULL)
		{
			alt = getGearAlt(gdata.at(currentIndex).vessel);
			spd = gdata.at(currentIndex).vessel->GetAirspeed();
			//sprintf(oapiDebugString(),"%f",alt);

		}
		double distNxtWp = MYMATH::getDist(gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).lat,gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).lon);
		double radiusNxtWp = gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).radius;

		if (distNxtWp <= radiusNxtWp)
		{
			if (gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).type == TURNING_POINT)
			{
				if (gdata.at(currentIndex).route.size() > gdata.at(currentIndex).selectedWp+1 ) 
				{
					gdata.at(currentIndex).selectedWp++;			
				}
				else gdata.at(currentIndex).route.clear();
			}

			else if (gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).type == LOOPBACK_POINT)
			{
				gdata.at(currentIndex).selectedWp = gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).loopBackTo;
			}
			else if (gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).type == TAKEOFF_POINT)
			{
				if (alt > 20)
					if (gdata.at(currentIndex).route.size() > gdata.at(currentIndex).selectedWp+1 ) 
						gdata.at(currentIndex).selectedWp++;		
			}
			else if (gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).type == LANDING_POINT)
			{
				if (alt < 20 && spd < 0.0001)
					if (gdata.at(currentIndex).route.size() > gdata.at(currentIndex).selectedWp+1 ) 
						gdata.at(currentIndex).selectedWp++;		
			}

		}

		if(radiusNxtWp == 0)
		{
			if (gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).type == TAKEOFF_POINT)
			{
				if (alt > 20)
					if (gdata.at(currentIndex).route.size() > gdata.at(currentIndex).selectedWp+1 ) 
						gdata.at(currentIndex).selectedWp++;
			}
			else if (gdata.at(currentIndex).route.at(gdata.at(currentIndex).selectedWp).type == LANDING_POINT)
			{
				if (alt < 0.0001 && spd < 0.0001)
					if (gdata.at(currentIndex).route.size() > gdata.at(currentIndex).selectedWp+1 ) 
						gdata.at(currentIndex).selectedWp++;		
			}
		}
	}
}

Mymod *mymod;

DLLCLBK void InitModule (HINSTANCE hDLL)
{	
	static char *name = "WayPointMFD";
	MFDMODESPECEX spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_W;
	spec.context = NULL;
	spec.msgproc = WaypointMFD::MsgProc;
	mode = oapiRegisterMFDMode (spec);
	mymod = new Mymod (hDLL);
	oapiRegisterModule (mymod);
	
}

DLLCLBK void ExitModule (HINSTANCE hDLL)
{
	delete mymod;
}

//DLL- Unregister
DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
   oapiUnregisterMFDMode (mode);
   
}

/*
DLLCLBK void clbkSimulationEnd ()
{	
}

DLLCLBK void clbkSimulationStart ()
{
}

DLLCLBK void opcLoadState (FILEHANDLE scn)
{
}



DLLCLBK void opcPostStep (double ENow, double simdt, double mjd) 
{	


}
*/