#include "orbitersdk.h"
#include "CelBodyAPI.h"
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <vector>
#include <string>
#include <sstream>

#define TURNING_POINT 0
#define LOOPBACK_POINT 1
#define TAKEOFF_POINT 2
#define LANDING_POINT 3

using namespace std;

struct waypoint
{
	double lat;
	double lon;
	double alt;
	double radius;
	double speed;
	bool is3d;
	int type;
	string name;
	string comment;
	int loopBackTo;
};
struct point
{
	double lat,lon;
};

struct runway
{
	point p1;
	point p2;
	point center;
};

class SINGLE_MFDSETTINGS
{
	public:
		unsigned int mfdIdentifier;
		int mode;
		double zoom;
		unsigned long int virtualZoom;
};

class MFDHANDLE
{
	public:
		MFDHANDLE(){}
		std::vector<waypoint> route;
		vector<runway> runways;
		vector<SINGLE_MFDSETTINGS> smfds;
		unsigned int selectedWp;
		VESSEL* vessel;
		OBJHANDLE vh;
		unsigned int selectedFile;
		unsigned int maxfiles;
		bool flushFlag;
		int style;		
		const int getMode(unsigned int mfdIdentifier) const;
		int getMode(unsigned int mfdIdentifier);
		void setMode(unsigned int mfdIdentifier, int mode);
		double getZoom(unsigned int mfdIdentifier);
		unsigned long getVirtualZoom(unsigned int mfdIdentifier);
		void setZoom(unsigned int mfdIdentifier, double zoom);
		void setVirtualZoom(unsigned int mfdIdentifier, unsigned long int virtualZoom);
	private:		
		SINGLE_MFDSETTINGS *getMFDSignleData(unsigned int mfdIdentifier);
};

class drawnWaypoint
{
	public:
		int x;
		int y;
		int radius;
		int kind;
};

class WaypointMFD: public MFD2
{
	public:
		//functions
		WaypointMFD (DWORD w, DWORD h, VESSEL *vessel,unsigned int mfd);
		bool ConsumeKeyBuffered (DWORD key);
		bool WaypointMFD::Update (oapi::Sketchpad *skp);
		static int MsgProc (UINT msg, UINT mfinald, WPARAM wparam, LPARAM lparam);
		int ButtonMenu (const MFDBUTTONMENU **menu) const;
		char *ButtonLabel (int bt);
		bool ConsumeButton(int bt, int event);		
		void init();
	protected:
		int height;
		int width;
	private:
		char *font;
		MFDHANDLE hmfd; //THIS MFD MODE
		unsigned int mfdid;
		DWORD darkColor;
		DWORD stdColor;
		void getCurrentVesselIndex();
		void pushCurrentVesselIndex();
		int line( int );
		int col( int i );
		HDC MFD_HDC;
		void drawWaypoints(oapi::Sketchpad *skp);		
		void drawBases(oapi::Sketchpad *skp);
		void drawMap(oapi::Sketchpad *skp);
		std::vector<drawnWaypoint> drawnRoute;
		std::vector<drawnWaypoint> createDrawnRoute();
		drawnWaypoint createDrawnWaypoint(double x, double y,double radius);
		drawnWaypoint equPosToPixelPos(double lat,double lon,double radius=0);
		drawnWaypoint equPosToPixelPos(waypoint wp);
		drawnWaypoint equPosToPixelPos(point p);
		int offsetX;
		int offsetY;
		void drawShipSymbol(oapi::Sketchpad *skp);
		bool isInit;
		oapi::Pen *darkPenX;
		oapi::Pen *stdPenX;
		void loadTrack();
		void listFiles(oapi::Sketchpad *skp);
		std::vector<std::string> fileNames;
		void parseTrack();
		void drawVORs(oapi::Sketchpad *skp);
		void writeWpInfo(oapi::Sketchpad *skp);
		void drawRunways(oapi::Sketchpad *skp);
		void writeWpList(oapi::Sketchpad *skp);
		string parserErrorString;
		VESSEL* vessel;
};

class parser2
{
	public:
		parser2();
		parser2(string filename); //Opens the file
		void readFile(std::string fileName);
		std::vector<waypoint> getRoute(string fileName);
		vector<runway> getRunways()
		{
			return runways;
		}
		string getErrorString();
	private:
		string errorstring;
		vector<runway> getRunways(vector<point> points);
		string readFileLocal(const char* filename);
		string getElements(string elementName);
		string fileString;
		string replace(string source,string replace, string by);
		string replaceAll();
		vector<string> split(string sourceString,string delimitter);
		void errorOut(string errormessage);
		vector<point> getPoints();
		vector<waypoint> getRoute(vector<point> points, vector<runway> runways);
		int myatoi(string s);
		double myatof(string s);
		vector<waypoint> meineroute;
		vector<runway> runways;
		bool parsererror;
};

class MYMATH
{
	public:
		static double getDist(double b2, double l2);
		static double getDist(double b1, double l1, double b2, double l2, OBJHANDLE rbody = 0);
		static double MYMATH::degToRad(double deg);

};